package	jp.co.sds_corp.syslog.viewer; 

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ResourceBundle;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.swing.border.TitledBorder;

import jp.co.sds_corp.syslog.action.IAction;
import jp.co.sds_corp.syslog.layout.ILayout;
import jp.co.sds_corp.syslog.viewer.action.IActionPanel;
import jp.co.sds_corp.util.common.CmnLog;
import jp.co.sds_corp.util.window.WindowUtility;

/**
* {@link IAction}を設定するダイアログです。
*/
public class ActionDialog extends JDialog
{
	private static final long serialVersionUID = 1L;
	SyslogParam	param;
	Box			buttonBox;
	JButton		BTN_OK = new JButton( "OK" );
	JButton		BTN_CANCEL = new JButton( "Cancel" );
	IActionPanel actionPanel;
	FilterPanel	filterPanel;
	LayoutPanel	layoutPanel;
	Box			layoutBox;
	boolean		bOk;
	ResourceBundle res;

	IAction		action;

	public ActionDialog( JFrame parent, SyslogParam param ) {
		super( parent );
		this.param = param;
		filterPanel = new FilterPanel( parent, param );
		layoutPanel = new LayoutPanel( parent, param );

		createControls();
	}

	public ActionDialog( JDialog parent, SyslogParam param ) {
		super( parent );
		this.param = param;
		filterPanel = new FilterPanel( parent, param );
		layoutPanel = new LayoutPanel( parent, param );

		createControls();
	}

	protected void createControls() {
		setModal( true );
		setResizable( false );
		WindowUtility.setHideOnEscape( this );

		res = ResourceBundle.getBundle( "resource.viewer" );
		String text;
		text = res.getString( "action.title" );
		setTitle( text );
		setSize( 477, 421 );
		setFont( param.getDefaultFont() );
		getContentPane().setLayout( new BoxLayout( getContentPane(), BoxLayout.Y_AXIS ));

		text = res.getString( "action.filter.text" );
		filterPanel.setBorder( new TitledBorder( text ));
		layoutBox = new Box( BoxLayout.X_AXIS );
		text = res.getString( "action.layout.text" );
		layoutBox.setBorder( new TitledBorder( text ));
		layoutBox.add( layoutPanel );
		layoutBox.add( Box.createHorizontalStrut( 500 ));

		buttonBox = new Box( BoxLayout.X_AXIS );
		buttonBox.add( BTN_OK );
		buttonBox.add( BTN_CANCEL );

		BTN_OK.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				onOk();
			}
		});
		BTN_CANCEL.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				onCancel();
			}
		});

	}

	public IAction getAction() {
		return action;
	}

	public void setAction( IAction action ) {
		this.action = action;
	}

	public void clearAction() {
		this.action = null;
	}

	public void setVisible( boolean var ) {
		if ( !var ) {
			super.setVisible( false );
		}
		String msg, title;
		bOk = false;
		getContentPane().removeAll();
		try  {
			Box box = new Box( BoxLayout.Y_AXIS );
			actionPanel = param.getActionPanel( action );
			if ( actionPanel == null ) {
				title = res.getString( "action.err2.title" );
				msg = res.getString( "action.err2.message" );
				JOptionPane.showMessageDialog( this,
					msg, title,
					JOptionPane.ERROR_MESSAGE );
				return;
			}
			actionPanel.setAction( action );
			box.add( (JPanel )actionPanel );
			try {
				ILayout l = action.getLayout();
				layoutPanel.setSyslogLayout( l );
				box.add( layoutBox );
			} catch ( UnsupportedOperationException uex ) {
			}
			filterPanel.clearFilter();
			filterPanel.addFilterList( action.listFilters() );
			box.add( filterPanel );
			box.add( buttonBox );
			getContentPane().add( box );
			BTN_OK.requestFocus();
			getRootPane().setDefaultButton( BTN_OK );
			SwingUtilities.updateComponentTreeUI( this );
			super.setVisible( true );
		} catch ( Exception ex ) {
			CmnLog.error( this, "error", ex );
			title = res.getString( "action.err1.title" );
			msg = res.getString( "action.err1.message" );
			JOptionPane.showMessageDialog( this,
					msg + "\n" + ex, title,
					JOptionPane.ERROR_MESSAGE );
		}
	}

	public void onOk() {
		try {
			action.clearFilter();
			action.addFilterList( filterPanel.listFilters() );
			actionPanel.updateAction();
			layoutPanel.updateLayout();
			ILayout layout = layoutPanel.getSyslogLayout();
			try {
				action.setLayout( layout );
			} catch ( UnsupportedOperationException uex ) {
			}
		} catch ( Exception ex ) {
			CmnLog.error( this, "error " + ex );
			String title = res.getString( "action.err3.title" );
			String msg = res.getString( "action.err3.message" );
			if ( ex.getMessage() == null ) {
				msg += "\n" + ex.toString();
			} else {
				msg += "\n" + ex.getMessage();
			}
			JOptionPane.showMessageDialog( this,
				msg, title, JOptionPane.ERROR_MESSAGE );
			return;
		}
		bOk = true;
		setVisible( false );
	}

	public void onCancel() {
		setVisible( false );
	}

	public boolean isOk() {
		return bOk;
	}
}
