package	jp.co.sds_corp.syslog.viewer;

import java.awt.Component;
import java.awt.Window;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.ResourceBundle;
import java.util.Vector;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.AbstractTableModel;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumn;

import jp.co.sds_corp.syslog.action.IAction;
import jp.co.sds_corp.util.common.CmnLog;
import jp.co.sds_corp.util.window.WindowUtility;

/**
* {@link IAction}の設定パネルです。
*/
public class ActionPanel extends JPanel
{
	private static final long serialVersionUID = 1L;
	Window		owner;
	SyslogParam	param;
	Vector<IAction>	vecAction = new Vector<IAction>();
	ActionModel	model;
	JTable	table;
	JButton	BTN_ADD;
	JButton	BTN_EDIT;
	JButton	BTN_DEL;
	JButton	BTN_VAL;
	ActionDialog	dlg;
	boolean	bChg = false;
	ResourceBundle res;

	public ActionPanel( JFrame owner, SyslogParam param ) {
		this.owner = owner;
		this.param = param;
		dlg = new ActionDialog( owner, param );
		createControls();
	}

	public ActionPanel( JDialog owner, SyslogParam param ) {
		this.owner = owner;
		this.param = param;
		dlg = new ActionDialog( owner, param );
		createControls();
	}

	protected void createControls() {
		setLayout( new BoxLayout( this, BoxLayout.X_AXIS ));

		res = ResourceBundle.getBundle( "resource.viewer" );
		String text;
		model = new ActionModel();
		table = new JTable( model );
		WindowUtility.removeTableEscape( table );
		JScrollPane sp = new JScrollPane( table );
		add( sp );
		table.addMouseListener( new MouseAdapter() {
        	public void mouseClicked( MouseEvent e ) {
				if ( e.getClickCount() == 2 ) {
					onEdit();
				}
			}
		});
		table.addKeyListener( new KeyAdapter() {
			public void keyPressed( KeyEvent e ) {
				if ( e.getKeyChar() == '\n' ) {
					onEdit();
					e.consume();
				}
			}
		});
		table.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );
		TableColumn col;
		col = table.getColumnModel().getColumn( 0 );
		col.setPreferredWidth( 100 );
		col = table.getColumnModel().getColumn( 1 );
		col.setPreferredWidth( 400 );
		col.setCellRenderer( new DescRenderer() );
		// col.setPreferredWidth( 500 );
		// table.setAutoResizeMode( JTable.AUTO_RESIZE_OFF );

		JPanel pnl = new JPanel();
		pnl.setLayout( new BoxLayout( pnl, BoxLayout.Y_AXIS ));
		text = res.getString( "action.add.text" );
		BTN_ADD = new JButton( text );
		BTN_ADD.setMnemonic( KeyEvent.VK_A );
		pnl.add( BTN_ADD );
		text = res.getString( "action.edit.text" );
		BTN_EDIT = new JButton( text );
		BTN_EDIT.setMnemonic( KeyEvent.VK_E );
		pnl.add( BTN_EDIT );
		text = res.getString( "action.del.text" );
		BTN_DEL = new JButton( text );
		BTN_DEL.setMnemonic( KeyEvent.VK_D );
		pnl.add( BTN_DEL );
		text = res.getString( "action.valid.text" );
		BTN_VAL = new JButton( text );
		BTN_VAL.setMnemonic( KeyEvent.VK_V );
		pnl.add( BTN_VAL );
		add( pnl );
		BTN_ADD.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				onAdd();
			}
		});
		BTN_EDIT.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				onEdit();
			}
		});
		BTN_DEL.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				onDel();
			}
		});
		BTN_VAL.addActionListener( new ActionListener() {
			public void actionPerformed( ActionEvent e ) {
				onValid();
			}
		});

	}

	public void addActionList( java.util.List<IAction> lstAction ) {
		vecAction.addAll( lstAction );
		model.fireTableDataChanged();
	}
	public void addAction( IAction act ) {
		vecAction.add( act );
		int nRow = vecAction.size() - 1;
		model.fireTableRowsInserted( nRow, nRow );
	}
	public void clearAction() {
		bChg = false;
		vecAction.clear();
		table.clearSelection();
		model.fireTableDataChanged();
	}
	public java.util.List<IAction> listActions() {
		return vecAction;
	}

	protected void onAdd() {
		IAction a = createAction();
		if ( a == null ) {
			return;
		}
		dlg.clearAction();
		dlg.setAction( a );
		WindowUtility.center( this, dlg );
		dlg.setVisible( true );
		if ( dlg.isOk() ) {
			IAction action = dlg.getAction();
			vecAction.add( action );
			int nRow = vecAction.size() - 1;
			model.fireTableRowsInserted( nRow, nRow );
			bChg = true;
		}
	}

	private IAction createAction() {
		Map<String,SyslogParam.Registry> mapAction = param.getActionMap();
		String[] options = new String[ mapAction.size() ];
		int nPos = 0;
		HashMap<String,SyslogParam.Registry> mapSel = new HashMap<String,SyslogParam.Registry>();
		Iterator<String> itr = mapAction.keySet().iterator();
		while ( itr.hasNext() ) {
			String key = (String )itr.next();
			SyslogParam.Registry reg = (SyslogParam.Registry )mapAction.get( key );     
			mapSel.put( reg.getType(), reg );
			options[nPos++] = reg.getType();
		}
		String msg, title;
		title = res.getString( "action.add.title" );
		msg = res.getString( "action.add.message" );
		String sel = (String )JOptionPane.showInputDialog( owner,
			msg, title,
			JOptionPane.INFORMATION_MESSAGE,
			null, options, options[0] );
		if ( sel == null ) {
			return null;
		}

		String cls = ((SyslogParam.Registry )mapSel.get( sel )).getOrgClass();
		IAction ret = null;
		try {
			ret = (IAction )Class.forName( cls ).newInstance();
		} catch ( Exception ex ) {
			CmnLog.error( this, "create error", ex );
			title = res.getString( "action.add.err1.title" );
			msg = res.getString( "action.add.err1.message" );
			JOptionPane.showMessageDialog( this,
				msg + " [" + cls + "]\n" + ex, title,
				JOptionPane.ERROR_MESSAGE );
			return null;
		}
		return ret;
	}       

	protected void onEdit() {
		int nPos = table.getSelectedRow();
		if ( table.getRowCount() == 0 || nPos < 0 ) {
			return;
		}
		IAction action = (IAction )vecAction.get( nPos );
		dlg.setAction( action );
		WindowUtility.center( this, dlg );
		dlg.setVisible( true );
		if ( dlg.isOk()) {
			bChg = true;
			model.fireTableRowsUpdated( nPos, nPos );
		}
	}

	protected void onDel() {
		int nPos = table.getSelectedRow();
		if ( table.getRowCount() == 0 || nPos < 0 ) {
			return;
		}
		String msg, title;
		title = res.getString( "action.del.title" );
		msg = res.getString( "action.del.message" );
		int nRet = JOptionPane.showConfirmDialog( this,
			msg, title,
			JOptionPane.YES_NO_OPTION );
		if ( nRet != JOptionPane.YES_OPTION ) {
			return;
		}
		vecAction.remove( nPos );
		model.fireTableRowsDeleted( nPos, nPos );
		table.clearSelection();
		bChg = true;
	}

	protected void onValid() {
		int nPos = table.getSelectedRow();
		if ( table.getRowCount() == 0 || nPos < 0 ) {
			return;
		}
		IAction action = (IAction )vecAction.get( nPos );
		action.setValid( !action.isValid());
		model.fireTableRowsUpdated( nPos, nPos );
		bChg = true;
	}

	public void setChange( boolean var ) {
		this.bChg = var;
	}
	public boolean isChange() {
		return bChg;
	}

	class ActionModel extends AbstractTableModel
	{
		private static final long serialVersionUID = 1L;
		String[] cols = { "Type", "Name" };

		public int getColumnCount() {
			return cols.length;
		}
		public String getColumnName( int col ) {
			return cols[col];
		}

		public int getRowCount() {
			return vecAction.size();
		}
		public Object getValueAt( int row, int col ) {
			IAction action = (IAction )vecAction.get( row );
			switch ( col ) {
			case 0:
				SyslogParam.Registry reg = param.getActionType( action );
				if ( reg == null ) {
					return "(UNKNOWN)";
				}
				return reg.getType();
			case 1: return action.paramString();
			}
			return null;
		}
	}

	public static class DescRenderer extends DefaultTableCellRenderer
	{
		private static final long serialVersionUID = 1L;

		public Component getTableCellRendererComponent(
			JTable table, Object obj, boolean isSelected, boolean hasFocus,
			int row, int column)
		{
			JLabel lbl = (JLabel )super.getTableCellRendererComponent(
							table, obj, isSelected, hasFocus, row, column );
			lbl.setToolTipText( obj.toString() );
			return lbl;
		}
	}

}

